#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <Pt.h>

#ifdef __USAGE
%C - transports a photon application

%C -nnode -ppid|-rrid

-n node  Node to transport to.
-p pid   Process whose windows are to be transported.
-r rid   Application region of an application, all of the application's windows
         will be transported.

Examples:
phtransport -n//96/dev/photon 122

phtransport -n96 122
#endif

char    *progname = "phtransport";

int
get_digits( char *str )
{
    char    *p;
    
    for ( p = str; *p != NULL ; p++ )
        if (!isdigit(*p))
            return -1;
    return strtol(str, NULL, 0);
}

PhRid_t
get_a_window_rid( pid_t pid )
{
    PhRegion_t  rinfo;
    PhRid_t     rids[100];
    int         n;

    /*
     * Walk through the window manager's frame regions looking for any that
     * have child regions that are owned by the given process.  For each
     * PtWindow region created, there are two regions: window manager
     * frame region (Ph_WND_MGR_REGION) and application region.  The
     * frame region is owned by the window manager and the application
     * region is owned by the application.  The application region is
     * a child of the window frame region.  It is the application region
     * that we are after.
     */
     
    n = PhQueryRids( 0, 0, 0, Ph_WND_MGR_REGION, ~0, 0, NULL, rids, 100 );
    while ( n > 0 ) {
        n--;
        PhRegionQuery( rids[n], &rinfo, NULL, NULL, 0 );
        if ( rinfo.child != -1 ) {
            PhRegionQuery( rinfo.child, &rinfo, NULL, NULL, 0 );
            if ( rinfo.owner == pid )
                return rinfo.rid;
        }
    }
    return -1;
}

int
transport_rid( char *device, PhRid_t rid )
{
    PhEvent_t   event;
    PhRect_t    rect = { 0, 0, 0, 0 };

    memset( &event, 0, sizeof(event) );
    event.type = Ph_EV_WM;
    event.subtype = Ph_EV_WM_TRANSPORT;
    event.emitter.rid = Ph_DEV_RID;
    event.collector.rid = rid;
    event.flags = Ph_EVENT_DIRECT;
    event.num_rects = 1;
    event.data_len =  strlen( device ) + 1;
    PhEventEmit( &event, &rect, device );
    
    return 0;
}

int
transport_pid( char *device, pid_t pid )
{
    PhRid_t rid;

    /*
     * Get the application region of any of this process's windows.
     * Transporting any window of an application transports them all.
     */
     
    if ((rid = get_a_window_rid( pid )) == -1)
        return -1;
    return transport_rid( device, rid );
}

int
main( int argc, char *argv[] )
{
    char    *node = NULL, buf[PATH_MAX+1];
    int     c;
    pid_t   pid = -1;
    PhRid_t rid = -1;

    PtInit( NULL );

    while ((c = getopt(argc, argv, "n:p:r:")) != -1) {
        switch (c) {
        case 'n': node = optarg; break;
        case 'p': pid = strtol(optarg, NULL, 0); break;
        case 'r': rid = strtol(optarg, NULL, 0); break;
        case '?':
            exit( EXIT_FAILURE );
        }
    }
                                                                               
    if ( node == NULL ) {
        fprintf( stderr, "%s: Missing node to transport to (-n option)\n", progname );
        exit( EXIT_FAILURE );
    } else if ( pid == -1  &&  rid == -1 ) {
        fprintf( stderr, "%s: Missing pid or rid (-p or -r option)\n", progname );
        exit( EXIT_FAILURE );
    }
    
    if ( (c = get_digits(node)) != -1 ) {
        sprintf( buf, "//%d/dev/photon", c );
        node = buf;
    } else if ( node[0] != '/' ) {
        sprintf( buf, "/dev/%s", node );
        node = buf;
    } /* else fullpath given */
    
    if ( pid != -1 ) {
        transport_pid( node, pid );
    } else {
        transport_rid( node, rid );
    }
                                                                               
    return( 0 );
}
